@@ -43,7 +43,6 @@ from django.core.urlresolvers import reverse
 from django.db import models
 from django.db.models.fields import FieldDoesNotExist
 from django.db.models.signals import post_delete
-from django.db.models.sql.aggregates import Aggregate as SQLAggregate
 from django.dispatch import receiver
 from django.template import Template, Context
 from django.template.defaultfilters import filesizeformat
@@ -1529,148 +1528,6 @@ def file_cleanup(sender, instance, **kwargs):
             field.storage.delete(field.path)
 
 
-class FilterMatch(object):
-    """A non-database object that represents the way a filter matches a test_run.
-
-    Returned by TestRunFilter.matches_against_bundle and
-    TestRunFilter.get_test_runs.
-    """
-
-    filter = None
-    tag = None # either a date (bundle__uploaded_on) or a build number
-    test_runs = None
-    specific_results = None # Will stay none unless filter specifies a test case
-    pass_count = None # Only filled out for filters that dont specify a test
-    result_code = None # Ditto
-
-    def _format_test_result(self, result):
-        prefix = result.test_case.test.test_id + ':' + result.test_case.test_case_id + ' '
-        if result.test_case.units:
-            return prefix + '%s%s' % (result.measurement, result.units)
-        else:
-            return prefix + result.RESULT_MAP[result.result]
-
-    def _format_test_run(self, tr):
-        return "%s %s pass / %s total" % (
-            tr.test.test_id,
-            tr.denormalization.count_pass,
-            tr.denormalization.count_all())
-
-    def _format_many_test_runs(self):
-        return "%s pass / %s total" % (self.pass_count, self.result_count)
-
-    def format_for_mail(self):
-        r = [' ~%s/%s ' % (self.filter.owner.username, self.filter.name)]
-        if not self.filter_data['tests']:
-            r.append(self._format_many_test_runs())
-        else:
-            for test in self.filter_data['tests']:
-                if not test.all_case_ids():
-                    for tr in self.test_runs:
-                        if tr.test == test.test:
-                            r.append('\n    ')
-                            r.append(self._format_test_run(tr))
-                for case_id in test.all_case_ids():
-                    for result in self.specific_results:
-                        if result.test_case.id == case_id:
-                            r.append('\n    ')
-                            r.append(self._format_test_result(result))
-        r.append('\n')
-        return ''.join(r)
-
-
-class MatchMakingQuerySet(object):
-    """Wrap a QuerySet and construct FilterMatchs from what the wrapped query
-    set returns.
-
-    Just enough of the QuerySet API to work with DataTable (i.e. pretend
-    ordering and real slicing)."""
-
-    model = TestRun
-
-    def __init__(self, queryset, filter_data, prefetch_related):
-        self.queryset = queryset
-        self.filter_data = filter_data
-        self.prefetch_related = prefetch_related
-        if filter_data['build_number_attribute']:
-            self.key = 'build_number'
-            self.key_name = 'Build'
-        else:
-            self.key = 'bundle__uploaded_on'
-            self.key_name = 'Uploaded On'
-
-    def _makeMatches(self, data):
-        test_run_ids = set()
-        for datum in data:
-            test_run_ids.update(datum['id__arrayagg'])
-        r = []
-        trs = TestRun.objects.filter(id__in=test_run_ids).select_related(
-            'denormalization', 'bundle', 'bundle__bundle_stream', 'test').prefetch_related(
-            *self.prefetch_related)
-        trs_by_id = {}
-        for tr in trs:
-            trs_by_id[tr.id] = tr
-        case_ids = set()
-        for t in self.filter_data['tests']:
-            case_ids.update(t.all_case_ids())
-        if case_ids:
-            result_ids_by_tr_id = {}
-            results_by_tr_id = {}
-            values = TestResult.objects.filter(
-                test_case__id__in=case_ids,
-                test_run__id__in=test_run_ids).values_list(
-                'test_run__id', 'id')
-            result_ids = set()
-            for v in values:
-                result_ids_by_tr_id.setdefault(v[0], []).append(v[1])
-                result_ids.add(v[1])
-
-            results_by_id = {}
-            for result in TestResult.objects.filter(
-                id__in=list(result_ids)).select_related(
-                'test', 'test_case', 'test_run__bundle__bundle_stream'):
-                results_by_id[result.id] = result
-
-            for tr_id, result_ids in result_ids_by_tr_id.items():
-                rs = results_by_tr_id[tr_id] = []
-                for result_id in result_ids:
-                    rs.append(results_by_id[result_id])
-        for datum in data:
-            trs = []
-            for id in set(datum['id__arrayagg']):
-                trs.append(trs_by_id[id])
-            match = FilterMatch()
-            match.test_runs = trs
-            match.filter_data = self.filter_data
-            match.tag = datum[self.key]
-            if case_ids:
-                match.specific_results = []
-                for id in set(datum['id__arrayagg']):
-                    match.specific_results.extend(results_by_tr_id.get(id, []))
-            else:
-                match.pass_count = sum(tr.denormalization.count_pass for tr in trs)
-                match.result_count = sum(tr.denormalization.count_all() for tr in trs)
-            r.append(match)
-        return iter(r)
-
-    def _wrap(self, queryset, **kw):
-        return self.__class__(queryset, self.filter_data, self.prefetch_related, **kw)
-
-    def order_by(self, *args):
-        # the generic tables code calls this even when it shouldn't...
-        return self
-
-    def count(self):
-        return self.queryset.count()
-
-    def __getitem__(self, item):
-        return self._wrap(self.queryset[item])
-
-    def __iter__(self):
-        data = list(self.queryset)
-        return self._makeMatches(data)
-
-
 class TestRunFilterAttribute(models.Model):
 
     name = models.CharField(max_length=1024)
@@ -1689,12 +1546,6 @@ class TestRunFilterTest(models.Model):
     index = models.PositiveIntegerField(
         help_text = _(u"The index of this test in the filter"))
 
-    def all_case_ids(self):
-        return self.cases.all().order_by('index').values_list('test_case__id', flat=True)
-
-    def all_case_names(self):
-        return self.cases.all().order_by('index').values_list('test_case__test_case_id', flat=True)
-
     def __unicode__(self):
         return unicode(self.test)
 
@@ -1710,22 +1561,6 @@ class TestRunFilterTestCase(models.Model):
         return unicode(self.test_case)
 
 
-class SQLArrayAgg(SQLAggregate):
-    sql_function = 'array_agg'
-
-
-class ArrayAgg(models.Aggregate):
-    name = 'ArrayAgg'
-    def add_to_query(self, query, alias, col, source, is_summary):
-        aggregate = SQLArrayAgg(
-            col, source=source, is_summary=is_summary, **self.extra)
-        # For way more detail than you want about what this next line is for,
-        # see
-        # http://voices.canonical.com/michael.hudson/2012/09/02/using-postgres-array_agg-from-django/
-        aggregate.field = models.DecimalField() # vomit
-        query.aggregates[alias] = aggregate
-
-
 class TestRunFilter(models.Model):
 
     owner = models.ForeignKey(User)
@@ -1757,86 +1592,24 @@ class TestRunFilter(models.Model):
         User, null=True, blank=True, related_name='+',
         help_text="Only consider bundles uploaded by this user")
 
-    @property
-    def summary_data(self):
+    def as_data(self):
+        tests = []
+        for trftest in self.tests.order_by('index').prefetch_related('cases'):
+            tests.append({
+                'test': trftest.test,
+                'test_cases': [trftestcase.test_case for trftestcase in trftest.cases.all().select_related('test_case')],
+                })
         return {
             'bundle_streams': self.bundle_streams.all(),
             'attributes': self.attributes.all().values_list('name', 'value'),
-            'tests': self.tests.all().prefetch_related('cases'),
+            'tests': tests,
             'build_number_attribute': self.build_number_attribute,
+            'uploaded_by': self.uploaded_by,
             }
 
     def __unicode__(self):
         return "<TestRunFilter ~%s/%s>" % (self.owner.username, self.name)
 
-    # given filter:
-    # select from testrun
-    #  where testrun.bundle in filter.bundle_streams ^ accessible_bundles
-    #    and testrun has attribute with key = key1 and value = value1
-    #    and testrun has attribute with key = key2 and value = value2
-    #    and               ...
-    #    and testrun has attribute with key = keyN and value = valueN
-    #    and testrun has any of the tests/testcases requested
-
-    def get_test_runs_impl(self, user, bundle_streams, attributes, tests, prefetch_related=[]):
-        accessible_bundle_streams = BundleStream.objects.accessible_by_principal(
-            user)
-        bs_ids = [bs.id for bs in set(accessible_bundle_streams) & set(bundle_streams)]
-        conditions = [models.Q(bundle__bundle_stream__id__in=bs_ids)]
-
-        content_type_id = ContentType.objects.get_for_model(TestRun).id
-
-        for (name, value) in attributes:
-            # We punch through the generic relation abstraction here for 100x
-            # better performance.
-            conditions.append(
-                models.Q(id__in=NamedAttribute.objects.filter(
-                    name=name, value=value, content_type_id=content_type_id
-                    ).values('object_id')))
-
-        test_condition = None
-        for test in tests:
-            cases = list(test.all_case_ids())
-            if cases:
-                q = models.Q(
-                    test__id=test.test.id,
-                    test_results__test_case__id__in=cases)
-            else:
-                q = models.Q(test__id=test.test.id)
-            if test_condition:
-                test_condition = test_condition | q
-            else:
-                test_condition = q
-        if test_condition:
-            conditions.append(test_condition)
-
-        if self.uploaded_by:
-            conditions.append(models.Q(bundle__uploaded_by=self.uploaded_by))
-
-        testruns = TestRun.objects.filter(*conditions)
-
-        if self.build_number_attribute:
-            testruns = testruns.filter(
-                attributes__name=self.build_number_attribute).extra(
-                select={
-                    'build_number': 'convert_to_integer("dashboard_app_namedattribute"."value")',
-                    },
-                where=['convert_to_integer("dashboard_app_namedattribute"."value") IS NOT NULL']).extra(
-                order_by=['-build_number'],
-                ).values('build_number').annotate(ArrayAgg('id'))
-        else:
-            testruns = testruns.order_by('-bundle__uploaded_on').values(
-                'bundle__uploaded_on').annotate(ArrayAgg('id'))
-
-        filter_data = {
-            'bundle_streams': bundle_streams,
-            'attributes': attributes,
-            'tests': tests,
-            'build_number_attribute': self.build_number_attribute,
-            }
-
-        return MatchMakingQuerySet(testruns, filter_data, prefetch_related)
-
     # given bundle:
     # select from filter
     #  where bundle.bundle_stream in filter.bundle_streams
@@ -1848,6 +1621,7 @@ class TestRunFilter(models.Model):
 
     @classmethod
     def matches_against_bundle(self, bundle):
+        from dashboard_app.filters import FilterMatch
         bundle_filters = bundle.bundle_stream.testrunfilter_set.all()
         attribute_filters = bundle_filters.extra(
             where=[
@@ -1901,14 +1675,6 @@ class TestRunFilter(models.Model):
             matches.append(match)
         return matches
 
-    def get_test_runs(self, user, prefetch_related=[]):
-        return self.get_test_runs_impl(
-            user,
-            self.bundle_streams.all(),
-            self.attributes.values_list('name', 'value'),
-            self.tests.all(),
-            prefetch_related)
-
     @models.permalink
     def get_absolute_url(self):
         return (
@@ -1960,7 +1726,7 @@ class TestRunFilterSubscription(models.Model):
                     failure_found = match.pass_count != match.result_count
                 else:
                     for t in match.filter_data['tests']:
-                        if not t.all_case_ids():
+                        if not t['test_cases']:
                             for tr in match.test_runs:
                                 if tr.test == t.test:
                                     if tr.denormalization.count_pass != tr.denormalization.count_all():
