@@ -421,8 +421,7 @@ class TestTable(DataTablesTable):
     result = TemplateColumn('''
         <a href="{{record.get_absolute_url}}">
           <img src="{{ STATIC_URL }}dashboard_app/images/icon-{{ record.result_code }}.png"
-          alt="{{ record.get_result_display }}" width="16" height="16" border="0"/></a>
-        <a href="{{record.get_absolute_url}}">{{ record.get_result_display }}</a>
+          alt="{{ record.result_code }}" width="16" height="16" border="0"/>{{ record.result_code }}
         {% if record.attachments__count %}
         <a href="{{record.get_absolute_url}}#attachments">
           <img style="float:right;" src="{{ STATIC_URL }}dashboard_app/images/attachment.png"
